
/*****
 ForgetMeNot, Smart Outlet
*****/

#include <ESP8266WiFi.h>
#include <PubSubClient.h>

// Change the credentials below, so your ESP8266 connects to your network. USE A HOTSPOT! Proper WiFi does not work for some reason
const char* ssid = "<WIFI_SSID>";
const char* password = "<WIFI_PASSWORD>";

// Run Mosquitto, do "ifconfig" and get the IP address of your laptop on the HOTSPOT's interface, and add it here
const char* mqtt_server = "<YOUR_LOCAL_MQTT_BROKER_IP>";
 
// Initializes the espClient
WiFiClient espClient;
PubSubClient client(espClient);

// Set pins
#define OUTPUT_PIN D1

// This function connects your ESP8266 to your network
void setup_wifi() {
  delay(10);
  // We start by connecting to a WiFi network
  Serial.begin(9600);
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.print("WiFi connected - ESP IP address: ");
  Serial.println(WiFi.localIP());
}

void callback(String topic, byte* message, unsigned int length) {
  String messageTemp;
  for (int i = 0; i < length; i++) {
    Serial.print((char)message[i]);
    messageTemp += (char)message[i];
  }
  Serial.println();

  // If a message is received on the topic house/switch, you check if the message is either on or off. 
  //Turns the Smart Outlet pin according to the message
  if(topic=="house/switch"){
      if(messageTemp == "on"){
        digitalWrite(OUTPUT_PIN, HIGH);
      }
      else if(messageTemp == "off"){
       digitalWrite(OUTPUT_PIN, LOW);
      }
  }
}

void reconnect() {
  // Loop until we're reconnected
  while (!client.connected()) {
    Serial.print("Attempting MQTT connection...");
    // Attempt to connect
    if (client.connect("ESP8266SmartOutlet")) {
      Serial.println("connected");  
      client.subscribe("house/switch");
    } else {
      Serial.print("failed, rc=");
      Serial.print(client.state());
      Serial.println(" try again in 5 seconds");
      // Wait 5 seconds before retrying
      delay(5000);
    }
  }
}

void setup() {
  pinMode(OUTPUT_PIN, OUTPUT);
  Serial.begin(9600);
  setup_wifi();
  client.setServer(mqtt_server, 1883);
  client.setCallback(callback);
}

void loop() {
   if (!client.connected()) {
    reconnect();
  }
  if(!client.loop()) {
    client.connect("ESP8266SmartOutlet");
  }

}
